@@ -11,25 +11,25 @@ Spyder kernel for Jupyter.
 """
 
 # Standard library imports
-from distutils.version import LooseVersion
 import logging
 import os
 import sys
 import threading
 
 # Third-party imports
-import ipykernel
 from ipykernel.ipkernel import IPythonKernel
+from ipykernel import eventloops
 from traitlets.config.loader import LazyConfigValue
 
 # Local imports
 from spyder_kernels.py3compat import (
-    TEXT_TYPES, to_text_string, PY3, input, TimeoutError)
-from spyder_kernels.comms.frontendcomm import FrontendComm, CommError
+    TEXT_TYPES, to_text_string, PY3)
+from spyder_kernels.comms.frontendcomm import FrontendComm
 from spyder_kernels.utils.iofuncs import iofunctions
 from spyder_kernels.utils.mpl import (
     MPL_BACKENDS_FROM_SPYDER, MPL_BACKENDS_TO_SPYDER, INLINE_FIGURE_FORMATS)
-from spyder_kernels.utils.nsview import get_remote_data, make_remote_view
+from spyder_kernels.utils.nsview import (
+    get_remote_data, make_remote_view, get_size)
 from spyder_kernels.console.shell import SpyderShell
 
 if PY3:
@@ -87,7 +87,6 @@ class SpyderKernel(IPythonKernel):
             'pdb_input_reply': self.pdb_input_reply,
             '_interrupt_eventloop': self._interrupt_eventloop,
             'enable_faulthandler': self.enable_faulthandler,
-            "flush_std": self.flush_std,
             }
         for call_id in handlers:
             self.frontend_comm.register_call_handler(
@@ -96,7 +95,6 @@ class SpyderKernel(IPythonKernel):
         self.namespace_view_settings = {}
         self._mpl_backend_error = None
         self._running_namespace = None
-        self._pdb_input_line = None
         self.faulthandler_handle = None
 
     # -- Public API -----------------------------------------------------------
@@ -120,11 +118,6 @@ class SpyderKernel(IPythonKernel):
             callback=callback,
             timeout=timeout)
 
-    def flush_std(self):
-        """Flush C standard streams."""
-        sys.__stderr__.flush()
-        sys.__stdout__.flush()
-
     def enable_faulthandler(self, fn):
         """
         Open a file to save the faulthandling and identifiers for
@@ -210,9 +203,9 @@ class SpyderKernel(IPythonKernel):
             properties = {}
             for name, value in list(data.items()):
                 properties[name] = {
-                    'is_list':  isinstance(value, (tuple, list)),
-                    'is_dict':  isinstance(value, dict),
-                    'is_set': isinstance(value, set),
+                    'is_list':  self._is_list(value),
+                    'is_dict':  self._is_dict(value),
+                    'is_set': self._is_set(value),
                     'len': self._get_len(value),
                     'is_array': self._is_array(value),
                     'is_image': self._is_image(value),
@@ -306,21 +299,6 @@ class SpyderKernel(IPythonKernel):
             return self.shell.pdb_session.do_complete(code, cursor_pos)
         return self._do_complete(code, cursor_pos)
 
-    def publish_pdb_state(self, step):
-        """
-        Publish Variable Explorer state and Pdb step through
-        send_spyder_msg.
-        """
-        state = dict(
-            namespace_view=self.get_namespace_view(),
-            var_properties=self.get_var_properties(),
-            step=step
-         )
-        try:
-            self.frontend_call(blocking=False).pdb_state(state)
-        except (CommError, TimeoutError):
-            logger.debug("Could not send Pdb state to the frontend.")
-
     def set_spyder_breakpoints(self, breakpoints):
         """
         Handle a message from the frontend
@@ -352,60 +330,19 @@ class SpyderKernel(IPythonKernel):
 
     def pdb_input_reply(self, line, echo_stack_entry=True):
         """Get a pdb command from the frontend."""
-        if self.shell.pdb_session:
-            self.shell.pdb_session._disable_next_stack_entry = (
-                not echo_stack_entry)
-        self._pdb_input_line = line
+        debugger = self.shell.pdb_session
+        if debugger:
+            debugger._disable_next_stack_entry = not echo_stack_entry
+            debugger._cmd_input_line = line
         if self.eventloop:
             # Interrupting the eventloop is only implemented when a message is
             # received on the shell channel, but this message is queued and
             # won't be processed because an `execute` message is being
-            # processed. Therefore we process the message here (comm channel)
+            # processed. Therefore we process the message here (control chan.)
             # and request a dummy message to be sent on the shell channel to
             # stop the eventloop. This will call back `_interrupt_eventloop`.
             self.frontend_call().request_interrupt_eventloop()
 
-    def cmd_input(self, prompt=''):
-        """
-        Special input function for commands.
-        Runs the eventloop while debugging.
-        """
-        # Only works if the comm is open and this is a pdb prompt.
-        if not self.frontend_comm.is_open() or not self.shell.is_debugging():
-            return input(prompt)
-
-        # Flush output before making the request.
-        sys.stderr.flush()
-        sys.stdout.flush()
-
-        # Send the input request.
-        self._pdb_input_line = None
-        self.frontend_call().pdb_input(prompt)
-
-        # Allow GUI event loop to update
-        if PY3:
-            is_main_thread = (
-                threading.current_thread() is threading.main_thread())
-        else:
-            is_main_thread = isinstance(
-                threading.current_thread(), threading._MainThread)
-
-        # Get input by running eventloop
-        if is_main_thread and self.eventloop:
-            while self._pdb_input_line is None:
-                eventloop = self.eventloop
-                if eventloop:
-                    eventloop(self)
-                else:
-                    break
-
-        # Get input by blocking
-        if self._pdb_input_line is None:
-            self.frontend_comm.wait_until(
-                lambda: self._pdb_input_line is not None)
-
-        return self._pdb_input_line
-
     def _interrupt_eventloop(self):
         """Interrupts the eventloop."""
         # Receiving the request is enough to stop the eventloop.
@@ -460,48 +397,43 @@ class SpyderKernel(IPythonKernel):
         """
         # Mapping from frameworks to backend names.
         mapping = {
-            'qt': 'QtAgg',  # For Matplotlib 3.5+
-            'qt5': 'Qt5Agg',
+            'qt': 'QtAgg',
             'tk': 'TkAgg',
             'macosx': 'MacOSX'
         }
 
-        try:
-            # --- Get interactive framework
-            framework = None
+        # --- Get interactive framework
+        framework = None
 
-            # This is necessary because _get_running_interactive_framework
-            # can't detect Tk in a Jupyter kernel.
-            if hasattr(self, 'app_wrapper'):
-                if hasattr(self.app_wrapper, 'app'):
-                    import tkinter
-                    if isinstance(self.app_wrapper.app, tkinter.Tk):
-                        framework = 'tk'
+        # Detect if there is a graphical framework running by checking the
+        # eventloop function attached to the kernel.eventloop attribute (see
+        # `ipykernel.eventloops.enable_gui` for context).
+        from IPython.core.getipython import get_ipython
+        loop_func = get_ipython().kernel.eventloop
 
-            if framework is None:
-                try:
-                    # This is necessary for Matplotlib 3.3.0+
-                    from matplotlib import cbook
-                    framework = cbook._get_running_interactive_framework()
-                except AttributeError:
-                    # For older versions
-                    from matplotlib import backends
-                    framework = backends._get_running_interactive_framework()
-
-            # --- Return backend according to framework
-            if framework is None:
-                # Since no interactive backend has been set yet, this is
-                # equivalent to having the inline one.
-                return 0
-            elif framework in mapping:
-                return MPL_BACKENDS_TO_SPYDER[mapping[framework]]
+        if loop_func is not None:
+            if loop_func == eventloops.loop_tk:
+                framework = 'tk'
+            elif loop_func == eventloops.loop_qt5:
+                framework = 'qt'
+            elif loop_func == eventloops.loop_cocoa:
+                framework = 'macosx'
             else:
-                # This covers the case of other backends (e.g. Wx or Gtk)
-                # which users can set interactively with the %matplotlib
-                # magic but not through our Preferences.
-                return -1
-        except Exception:
-            return None
+                # Spyder doesn't handle other backends
+                framework = 'other'
+
+        # --- Return backend according to framework
+        if framework is None:
+            # Since no interactive backend has been set yet, this is
+            # equivalent to having the inline one.
+            return 0
+        elif framework in mapping:
+            return MPL_BACKENDS_TO_SPYDER[mapping[framework]]
+        else:
+            # This covers the case of other backends (e.g. Wx or Gtk)
+            # which users can set interactively with the %matplotlib
+            # magic but not through our Preferences.
+            return -1
 
     def set_matplotlib_backend(self, backend, pylab=False):
         """Set matplotlib backend given a Spyder backend option."""
@@ -516,11 +448,7 @@ class SpyderKernel(IPythonKernel):
 
     def set_mpl_inline_resolution(self, resolution):
         """Set inline figure resolution."""
-        if LooseVersion(ipykernel.__version__) < LooseVersion('4.5'):
-            option = 'savefig.dpi'
-        else:
-            option = 'figure.dpi'
-        self._set_mpl_inline_rc_config(option, resolution)
+        self._set_mpl_inline_rc_config('figure.dpi', resolution)
 
     def set_mpl_inline_figure_size(self, width, height):
         """Set inline figure size."""
@@ -592,7 +520,6 @@ class SpyderKernel(IPythonKernel):
         try:
             import matplotlib.pyplot as plt
             plt.close('all')
-            del plt
         except:
             pass
 
@@ -634,11 +561,12 @@ class SpyderKernel(IPythonKernel):
                 sys.path.remove(path)
 
         # Add new paths
-        # We do this in reverse order as we use `sys.path.insert(1, path)`.
-        # This ensures the end result has the correct path order.
-        for path, active in reversed(new_path_dict.items()):
-            if active:
-                sys.path.insert(1, path)
+        pypath = [path for path, active in new_path_dict.items() if active]
+        if pypath:
+            sys.path.extend(pypath)
+            os.environ.update({'PYTHONPATH': os.pathsep.join(pypath)})
+        else:
+            os.environ.pop('PYTHONPATH', None)
 
     # -- Private API ---------------------------------------------------
     # --- For the Variable Explorer
@@ -650,17 +578,21 @@ class SpyderKernel(IPythonKernel):
         both locals() and globals() for current frame when debugging
         """
         ns = {}
-        if self._running_namespace is None:
+        if self.shell.is_debugging() and self.shell.pdb_session.curframe:
+            # Stopped at a pdb prompt
             ns.update(self.shell.user_ns)
+            ns.update(self.shell._pdb_locals)
         else:
-            # This is true when a file is executing.
-            running_globals, running_locals = self._running_namespace
-            ns.update(running_globals)
-            if running_locals is not None:
-                ns.update(running_locals)
+            # Give access to the running namespace if there is one
+            if self._running_namespace is None:
+                ns.update(self.shell.user_ns)
+            else:
+                # This is true when a file is executing.
+                running_globals, running_locals = self._running_namespace
+                ns.update(running_globals)
+                if running_locals is not None:
+                    ns.update(running_locals)
 
-        # Add debugging locals
-        ns.update(self.shell._pdb_locals)
         # Add magics to ns so we can show help about them on the Help
         # plugin
         if with_magics:
@@ -684,7 +616,7 @@ class SpyderKernel(IPythonKernel):
     def _get_len(self, var):
         """Return sequence length"""
         try:
-            return len(var)
+            return get_size(var)
         except:
             return None
 
@@ -720,6 +652,30 @@ class SpyderKernel(IPythonKernel):
         except:
             return False
 
+    def _is_list(self, var):
+        """Return True if variable is a list or tuple."""
+        # The try/except is necessary to fix spyder-ide/spyder#19516.
+        try:
+            return isinstance(var, (tuple, list))
+        except Exception:
+            return False
+
+    def _is_dict(self, var):
+        """Return True if variable is a dictionary."""
+        # The try/except is necessary to fix spyder-ide/spyder#19516.
+        try:
+            return isinstance(var, dict)
+        except Exception:
+            return False
+
+    def _is_set(self, var):
+        """Return True if variable is a set."""
+        # The try/except is necessary to fix spyder-ide/spyder#19516.
+        try:
+            return isinstance(var, set)
+        except Exception:
+            return False
+
     def _get_array_shape(self, var):
         """Return array's shape"""
         try:
