@@ -109,6 +109,17 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
         # Line received from the frontend
         self._cmd_input_line = None
 
+        # This is not available in IPython 5
+        if hasattr(self, '_predicates'):
+            # Turn off IPython's debugger skip funcionality by default because
+            # it makes our debugger quite slow. It's also important to remark
+            # that this functionality doesn't do anything on its own. Users
+            # need to mark what frames they want to skip for it to be useful.
+            # So, we hope that knowledgeable users will find that they need to
+            # enable it in Spyder.
+            # Fixes spyder-ide/spyder#20639.
+            self._predicates["debuggerskip"] = False
+
     # --- Methods overriden for code execution
     def print_exclamation_warning(self):
         """Print pdb warning for exclamation mark."""
@@ -186,7 +197,7 @@ class SpyderPdb(ipyPdb, object):  # Inherits `object` to call super() in PY2
 
                 code_ast = ast.parse(line)
 
-                if line.rstrip()[-1] == ";":
+                if line.rstrip()[-1:] == ";":
                     # Supress output with ;
                     capture_last_expression = False
                 else:
